//---------------------------------------------------------------------------
#include <vcl.h>
#include <stdio.h>

#pragma hdrstop

#include "fcmain.h"
#include "thdir.h"
#pragma package(smart_init)
//---------------------------------------------------------------------------
//   Important: Methods and properties of objects in VCL can only be
//   used in a method called using Synchronize, for example:
//
//      Synchronize(UpdateCaption);
//
//   where UpdateCaption could look like:
//
//      void __fastcall ThreadDir::UpdateCaption()
//      {
//        Form1->Caption = "Updated in a thread";
//      }
//---------------------------------------------------------------------------
__fastcall ThreadDir::ThreadDir(bool CreateSuspended)
    : TThread(CreateSuspended)
{

}
//---------------------------------------------------------------------------
//  Main thread source
//---------------------------------------------------------------------------
void __fastcall ThreadDir::Execute()
{
    //---- Place thread code here ----
    Kill = false;
    do {
        Synchronize(ThreadInit);
        SearchDirectoryTree (SearchPath);
        if (StopProcess) {
            Msg = "Process halted by user";
            Synchronize(UpdateThreadString);
            }
        Msg = "Found " + IntToStr((int)FileCount) + " file(s)";
        Synchronize(UpdateThreadString);
        Synchronize(ThreadSuspend);
        if (!Kill) {
            Suspend ();
            }
        } while (Terminated == false && Kill == false);
    ProcessRunning = false;
    Terminate ();
}
//---------------------------------------------------------------------------
void __fastcall ThreadDir::ThreadInit (void)
{
    Screen->Cursor = crHourGlass;
    FileCount   = 0;
    StopProcess = false;
    ProcessRunning = true;
    frm_fs->Animate1->Active = true;
    SelAttributes = frm_fs->RichEditSearch->SelAttributes;
}
//---------------------------------------------------------------------------
void __fastcall ThreadDir::ThreadSuspend (void)
{
    Screen->Cursor = crDefault;
    frm_fs->Animate1->Active = false;
    ProcessRunning = false;
    frm_fs->StatusBar1->SimpleText = "";
}
//---------------------------------------------------------------------------
void __fastcall ThreadDir::UpdateThreadString (void)
{
    frm_fs->RichEditSearch->SelAttributes = SelAttributes;
    frm_fs->RichEditSearch->Lines->Append (Msg);
    frm_fs->RichEditSearch->Perform (EM_SCROLLCARET, 0, 0);
    frm_fs->RichEditSearch->Perform (EM_SCROLL , SB_LINEUP, 0);
}
//---------------------------------------------------------------------------
void __fastcall ThreadDir::UpdateThreadStatusBar (void)
{
    frm_fs->StatusBar1->SimpleText = StatusMsg;

}
//---------------------------------------------------------------------------
//  SearchDirectoryTree ()  -   Scans directories with path as starting point.
//---------------------------------------------------------------------------
void ThreadDir::SearchDirectoryTree (String path)
{
    TSearchRec sr;
	int iAttributes = 0;
    String  WildCards;
    String  StrSearchPath;

try {
    if (StopProcess) return;

    iAttributes  = faDirectory;
    WildCards = ExtractFileName (path);
    if (ExtractFilePath (path).Length() > 0) {
        StrSearchPath = ExtractFilePath (path) + "*.*";
        }
    // ---- search for files in selected directory
    SearchDirectory (path);

    // ---- stop searching if not optional
    if (!SearchSubDir) return;

    // ---- scanning for new directories
	if (FindFirst(StrSearchPath, iAttributes, sr) == 0) {

		if (sr.Attr & faDirectory &&
            strstr (sr.Name.c_str(), ".") == NULL &&
            strstr (sr.Name.c_str(), "..") == NULL) {
            StatusMsg = "Scanning directory " + ExtractFilePath(path) + sr.Name;
            Synchronize(UpdateThreadStatusBar);
            SearchDirectoryTree(ExtractFilePath(path) + sr.Name + "\\" + WildCards);
            }
		while (FindNext(sr) == 0){
            if (sr.Attr & faDirectory &&
                strstr (sr.Name.c_str(), ".") == NULL &&
                strstr (sr.Name.c_str(), "..") == NULL) {
                StatusMsg = "Scanning directory " + ExtractFilePath(path) + sr.Name;
                Synchronize(UpdateThreadStatusBar);
                SearchDirectoryTree(ExtractFilePath(path) + sr.Name + "\\" + WildCards);
                }
		}
	}
	FindClose(sr);
    }
catch (Exception &exception)   {
    Application->ShowException(&exception);
    StopProcess = true;
    }
}
//---------------------------------------------------------------------------
// SearchDirectory ()   -   Search for files with wildcard specification in
//                          path
//---------------------------------------------------------------------------
void ThreadDir::SearchDirectory (String path)
{
try {
	int        iAttributes = 0;
    bool       FirstFound = true;

    iAttributes = faReadOnly | faHidden | faSysFile | faVolumeID | faArchive | faAnyFile;
	if (FindFirst(path, iAttributes, sr) == 0) {
        if (sr.Attr & iAttributes  &&
            strcmp (sr.Name.c_str(), ".") != 0 &&
            strcmp (sr.Name.c_str(), "..") != 0) {
            if (FirstFound) {
                FirstFound = false;
                SelAttributes->Color = clRed;
                if (ExtractFilePath (path).Length() == 0) {
                    Msg = ("<Directory> Current" );
                    Synchronize(UpdateThreadString);
                    }
                else {
                    Msg = ("<Directory> " + ExtractFilePath (path));
                    Synchronize(UpdateThreadString);
                    }
                }
                /*
            frm_fs->RichEditSearch->SelAttributes->Color = clBlack;
            frm_fs->RichEditSearch->Lines->Append ("  File : " + sr.Name + " " + IntToStr(sr.Size));
            LineCount++;
            */
            SearchFileText (ExtractFilePath (path) + sr.Name, SearchText);
            FileCount++;
    		}
		while (FindNext(sr) == 0) {
			if (sr.Attr & iAttributes  &&
                strcmp (sr.Name.c_str(), ".") != 0 &&
                strcmp (sr.Name.c_str(), "..") != 0) {
                if (FirstFound) {
                    FirstFound = false;
                    SelAttributes->Color = clRed;
                    if (ExtractFilePath (path).Length() == 0) {
                        Msg =  ("<Directory> Current" );
                        Synchronize(UpdateThreadString);
                        }
                    else {
                        Msg =  ("<Directory> " + ExtractFilePath (path));
                        Synchronize(UpdateThreadString);
                        }
                    }
                /*
                frm_fs->RichEditSearch->SelAttributes->Color = clBlack;
                frm_fs->RichEditSearch->Lines->Append ("  File : " + sr.Name + " " + IntToStr(sr.Size));
                LineCount++;
                */
                SearchFileText (ExtractFilePath (path) + sr.Name, SearchText);
                FileCount++;
			    }
		}
	}
	FindClose(sr);
    }
catch (Exception &exception)   {
    Application->ShowException(&exception);
    StopProcess = true;
    }
}
//---------------------------------------------------------------------------
//  SearchFileText ()   -   Scans input file for search string
//---------------------------------------------------------------------------
void ThreadDir::SearchFileText (String InputFile, String text)
{
    FILE *InFile;
    char  strfile[600];
    char  astrfile[5][600]; // ---- saving 5 lines of text
    long  tp, ix, fp, LineCount, size;
    bool  fts;

try {
    // ---- do nothing if no valid input
    if (InputFile.Length() == 0 || text.Length() == 0) {
        SelAttributes->Color = clBlack;
        size = SelAttributes->Size;
        SelAttributes->Size += 2;
        Msg =  ("  File : " + sr.Name + " " + IntToStr(sr.Size) + " Bytes");
        Synchronize(UpdateThreadString);
        SelAttributes->Size = size;
        return;
        }

    fts = false;
    if ((InFile = fopen(InputFile.c_str(), "rt")) == NULL) {
        // ---- can't open file
        return;
        }
    tp = 0;
    LineCount = 0;
    while (fgets (strfile, 600, InFile) != NULL) {
        LineCount++;
        strcpy (astrfile[tp], strfile);
        fp = tp;
        tp++;
        if (tp > 4) tp = 0;
        // ---- check for matching string
        if (strstr (UpperCase(strfile).c_str(), text.UpperCase().c_str()) != NULL) {
            if (!fts) {
                SelAttributes->Color = clBlack;
                SelAttributes->Style << fsBold;
                size = SelAttributes->Size;
                SelAttributes->Size += 2;
                Msg =  ("  File : " + sr.Name + " " + IntToStr(sr.Size) + " Bytes");
                Synchronize(UpdateThreadString);
                SelAttributes->Size = size;
                fts = true;
                }
            // ---- print result
            ix = tp;
            SelAttributes->Color = clNavy;
            Msg =  ("\t--- BEGIN Line " + IntToStr((int) LineCount));
            Synchronize(UpdateThreadString);
            LineCount++;
            do {
                if (ix == fp) {
                    SelAttributes->Color = clBlue;
                    }
                else {
                    SelAttributes->Color = clBlack;
                    }
                if (strlen (astrfile[ix]) > 0) {
                    astrfile[ix][strlen(astrfile[ix])-1] = NULL;
                    }
                Msg =  ("\t\t" + (String)astrfile[ix]);
                Synchronize(UpdateThreadString);
                LineCount++;
                ix++;
                if (ix > 4) ix = 0;
                }
            while (ix != tp);
            SelAttributes->Color = clBlack;
            for (ix=0; ix<4; ix++) {
                if (fgets (strfile, 600, InFile) == NULL) break;
                LineCount++;
                if (strlen (strfile) > 0) {
                    strfile[strlen(strfile)-1] = NULL;
                    }
                if (strstr (UpperCase(strfile).c_str(), text.UpperCase().c_str()) != NULL) {
                    SelAttributes->Color = clBlue;
                    }
                else {
                    SelAttributes->Color = clBlack;
                    }
                Msg =  ("\t\t" + (String)strfile);
                Synchronize(UpdateThreadString);
                LineCount++;
                }
            SelAttributes->Color = clNavy;
            Msg =  ("\t--- END");
            Synchronize(UpdateThreadString);
            LineCount++;
            }
        }
    fclose(InFile);
    /*
    if (!fts) {
        size = frm_fs->RichEditSearch->SelAttributes->Size;
        frm_fs->RichEditSearch->SelAttributes->Size += 2;
        frm_fs->RichEditSearch->SelAttributes->Color = clBlack;
        frm_fs->RichEditSearch->Lines->Append ("  File : " + sr.Name + " " + IntToStr(sr.Size));
        frm_fs->RichEditSearch->SelAttributes->Size = size;
        }
        */
    }
catch (Exception &exception)   {
    Application->ShowException(&exception);
    StopProcess = true;
    }
}